#include "mrk/lexer_internal.h"

mrk_err mrk_lexer_init(mrk_lexer **out) {
  MRK_CALLOC(out, 1, sizeof(mrk_lexer));

  return mrk_err_ok;
}

void mrk_lexer_open(mrk_lexer *lexer, const char *buf, size_t len) {
  lexer->buf.s = buf;
  lexer->buf.len = len;
  lexer->pos.line = 0;
  lexer->pos.buf_index = 0;
  lexer->token.start = 0;
  lexer->token.end = 0;
}

bool mrk_lexer_at_end(const mrk_lexer *lexer) {
  return (lexer->buf.len > 0 && lexer->pos.buf_index == lexer->buf.len) ||
         (lexer->buf.s[lexer->pos.buf_index] == '\0');
}

char mrk_lexer_advance(mrk_lexer *lexer) {
  if (mrk_lexer_at_end(lexer)) {
    return '\0';
  }

  char c = lexer->buf.s[lexer->pos.buf_index];

  // A newline is still part of the previous line, so if the last character was
  // a newline, we now go to the next line
  if (lexer->buf.s[lexer->pos.buf_index] == '\0') {
    lexer->pos.line++;
    lexer->pos.line_index = 0;
  } else {
    lexer->pos.line_index++;
  }

  lexer->pos.buf_index++;
  lexer->token.end++;

  return c;
}

char mrk_lexer_peek(mrk_lexer *lexer) {
  if (mrk_lexer_at_end(lexer)) {
    return '\0';
  }

  return lexer->buf.s[lexer->pos.buf_index];
}

void mrk_lexer_emit(mrk_token *out, mrk_lexer *lexer, mrk_token_type type) {
  out->type = type;
  out->start = lexer->token.start;
  out->start = lexer->token.end;

  lexer->token.start = lexer->token.end;
}

mrk_lexer_err mrk_lexer_next(mrk_token *out, mrk_lexer *lexer) {
  if (mrk_lexer_at_end(lexer)) {
    return mrk_lexer_err_done;
  }

  switch (mrk_lexer_advance(lexer)) {
  case '#':
    mrk_lexer_emit(out, lexer, mrk_token_type_pound);
    break;
  }

  return mrk_lexer_err_ok;
}
